/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef COMMANDS_H
#define COMMANDS_H

#include <qobject.h>
#include <qmap.h>
#include <qvaluelist.h>
#include <qguardedptr.h>

#include <message.h>
#include <connectioninfo.h>

namespace IRC
{

class Window;
class Frontend;

class ReplyHandler : public QObject
{
    Q_OBJECT
public:
    ReplyHandler( Window *parent, const char *name = 0 );

signals:
    void replyHandlerFinished( IRC::ReplyHandler *handler );

public slots:
    virtual void finished();

    virtual void handleNumericMessage( const IRC::Message &msg ) = 0;

protected:
    Window *m_window;
    Frontend *m_frontend;
};

class Command : public ReplyHandler
{
    Q_OBJECT
public:
    Command( Window *parent, const char *name = 0 );

    void sendMessage( const Message &msg );

    virtual void finished();

signals:
//    void sendOneWayMessage( const Message &msg );
    void sendMessage( const IRC::Message &msg, QObject *replyHandler );

    void commandFinished( IRC::Command *cmd );

public slots:
    virtual void start() = 0;

private:
    bool m_finished;
};

class MacroCommand : public Command
{
    Q_OBJECT
public:
    MacroCommand( Window *parent, const char *name = 0 );
    MacroCommand( const Message &messageToSend, Window *parent, const char *name = 0 );
    MacroCommand( const MessageList &messagesToSend, Window *parent, const char *name = 0 );

    virtual void start();

protected:
    MessageList m_messages;
};

class SimpleCommand : public MacroCommand
{
    Q_OBJECT
    Q_PROPERTY( bool usesTerminatingCommandSequence READ usesTerminatingCommandSequence )
public:
    SimpleCommand( Window *parent, const char *name = 0 );
    SimpleCommand( ReplyHandler *replyHandler, Window *parent, const char *name = 0 );
    SimpleCommand( const Message &messageToSend, ReplyHandler *replyHandler, Window *parent, const char *name = 0 );
    SimpleCommand( const MessageList &messagesToSend, ReplyHandler *replyHandler, Window *parent, const char *name = 0 );

    virtual void handleNumericMessage( const IRC::Message &msg );

    bool usesTerminatingCommandSequence() const;
signals:
    /* @internal */
    void internalRedirect( const IRC::Message &msg );

protected:
    void setReplyHandler( ReplyHandler *replyHandler );

    virtual void customEvent( QCustomEvent *ev );

private:
    QGuardedPtr<ReplyHandler> m_replyHandler;
};

class LoginCommand : public MacroCommand
{
    Q_OBJECT
public:
    LoginCommand( const QString &host, Window *parent, const char *name = 0 );

    virtual void handleNumericMessage( const IRC::Message &msg );

private:
    ConnectionInfo &m_connInfo;
};

class DefaultReplyHandler : public ReplyHandler
{
    Q_OBJECT
public:
    DefaultReplyHandler( Window *parent, const char *name = 0 );

    virtual void handleNumericMessage( const IRC::Message &msg );
};

class NamesReplyHandler : public DefaultReplyHandler
{
    Q_OBJECT
    Q_PROPERTY( bool usesTerminatingCommandSequence READ usesTerminatingCommandSequence )
public:
    struct ChannelInfo
    {
        ChannelInfo() : type( PublicChannel ) {}
        ChannelInfo( const QString &_name, ChannelType _type )
            : name( _name ), type( _type ) {}

        QString name;
        ChannelType type;

        bool operator<( const ChannelInfo &rhs ) const
        { return name < rhs.name; }
        bool operator==( const ChannelInfo &rhs ) const
        { return name == rhs.name; }
    };

    struct NickInfo
    {
        NickInfo() : isOp( false ) {}

        QString name; 
        bool isOp : 1; 
    }; 
    typedef QValueList<NickInfo> NickInfoList;

    // channel -> list of users
    typedef QMap<ChannelInfo, NickInfoList> ChannelUsersMap;

    NamesReplyHandler( const QStringList &channels, Window *parent, const char *name = 0 );
    NamesReplyHandler( const QString &channel, Window *parent, const char *name = 0 );

    virtual void handleNumericMessage( const IRC::Message &msg );

    // NAMES is properly sequenced and terminates upon RPL_ENDOFNAMES
    bool usesTerminatingCommandSequence() const { return true; }

signals:
    void channelUsers( const IRC::NamesReplyHandler::ChannelUsersMap &users );

private:
    void parseNamesReply( const IRC::Message &msg );

    void insertNicksIntoMap( const ChannelInfo &channel, const QStringList &nicks );

    void emitInfoAndFinish();

    ChannelUsersMap m_users;
    QStringList m_channels;
};

class NamesCommand : public SimpleCommand
{
    Q_OBJECT
public:
    NamesCommand( const QStringList &channels, Window *parent, const char *name = 0 );

signals:
    void channelUsers( const IRC::NamesReplyHandler::ChannelUsersMap &users );
};

class JoinCommand : public NamesCommand
{
    Q_OBJECT
public:
    JoinCommand( const JoinMessage::ChannelInfoList &channelInfo, Window *parent, const char *name = 0 );

    virtual void handleNumericMessage( const IRC::Message &msg );
};

class WhoisReplyHandler : public DefaultReplyHandler
{
    Q_OBJECT
    Q_PROPERTY( bool usesTerminatingCommandSequence READ usesTerminatingCommandSequence )
public:
    WhoisReplyHandler( Window *parent, const char *name = 0 );

    virtual void handleNumericMessage( const IRC::Message &msg );

    // WHOIS is properly sequenced and terminates upon RPL_ENDOFWHOIS
    bool usesTerminatingCommandSequence() const { return true; }

signals:
    void userDetailsAvailable( const IRC::Person &details );

private:
    void parseUser( const IRC::Message &msg );
    void parseChannels( const IRC::Message &msg );

    Person m_user;
};

class UserCommand : public SimpleCommand
{
    Q_OBJECT
public:
    UserCommand( const Message &message, Window *parent, const char *name = 0 );
};

} // namespace IRC

#endif // COMMANDS_H
/* vim: et sw=4 ts=4
 */
